from __future__ import unicode_literals
import io, datetime, sys

if sys.version_info[0] == 3:
    long = int
    unicode = str


def dumps(obj, sort_keys=False):
    fout = io.StringIO()
    dump(fout, obj, sort_keys=sort_keys)
    return fout.getvalue()


_escapes = {'\n': 'n', '\r': 'r', '\\': '\\', '\t': 't', '\b': 'b', '\f': 'f', '"': '"'}


def _escape_string(s):
    res = []
    start = 0

    def flush():
        if start != i:
            res.append(s[start:i])
        return i + 1

    i = 0
    while i < len(s):
        c = s[i]
        if c in '"\\\n\r\t\b\f':
            start = flush()
            res.append('\\' + _escapes[c])
        elif ord(c) < 0x20:
            start = flush()
            res.append('\\u%04x' % ord(c))
        i += 1

    flush()
    return '"' + ''.join(res) + '"'


def _escape_id(s):
    if any(not c.isalnum() and c not in '-_' for c in s):
        return _escape_string(s)
    return s


def _format_list(v):
    return '[{0}]'.format(', '.join(_format_value(obj) for obj in v))

# Formula from:
#   https://docs.python.org/2/library/datetime.html#datetime.timedelta.total_seconds
# Once support for py26 is dropped, this can be replaced by td.total_seconds()
def _total_seconds(td):
    return ((td.microseconds
             + (td.seconds + td.days * 24 * 3600) * 10**6) / 10.0**6)

def _format_value(v):
    if isinstance(v, bool):
        return 'true' if v else 'false'
    if isinstance(v, int) or isinstance(v, long):
        return unicode(v)
    if isinstance(v, float):
        return '{0:.17f}'.format(v)
    elif isinstance(v, unicode) or isinstance(v, bytes):
        return _escape_string(v)
    elif isinstance(v, datetime.datetime):
        offs = v.utcoffset()
        offs = _total_seconds(offs) // 60 if offs is not None else 0

        if offs == 0:
            suffix = 'Z'
        else:
            if offs > 0:
                suffix = '+'
            else:
                suffix = '-'
                offs = -offs
            suffix = '{0}{1:.02}{2:.02}'.format(suffix, offs // 60, offs % 60)

        if v.microsecond:
            return v.strftime('%Y-%m-%dT%H:%M:%S.%f') + suffix
        else:
            return v.strftime('%Y-%m-%dT%H:%M:%S') + suffix
    elif isinstance(v, list):
        return _format_list(v)
    else:
        raise RuntimeError(v)


def dump(fout, obj, sort_keys=False):
    tables = [((), obj, False)]

    while tables:
        name, table, is_array = tables.pop()
        if name:
            section_name = '.'.join(_escape_id(c) for c in name)
            if is_array:
                fout.write('[[{0}]]\n'.format(section_name))
            else:
                fout.write('[{0}]\n'.format(section_name))

        table_keys = sorted(table.keys()) if sort_keys else table.keys()
        new_tables = []
        for k in table_keys:
            v = table[k]
            if isinstance(v, dict):
                new_tables.append((name + (k,), v, False))
            elif isinstance(v, list) and v and all(isinstance(o, dict) for o in v):
                new_tables.extend((name + (k,), d, True) for d in v)
            elif v is None:
                # based on mojombo's comment: https://github.com/toml-lang/toml/issues/146#issuecomment-25019344
                fout.write(
                    '#{} = null  # To use: uncomment and replace null with value\n'.format(_escape_id(k)))
            else:
                fout.write('{0} = {1}\n'.format(_escape_id(k), _format_value(v)))

        tables.extend(reversed(new_tables))

        if tables:
            fout.write('\n')



import ast
import tokenize
import io
import collections
import itertools


def rewrite_imports_in_module(source, top_level_names, depth):
    source_lines = source.splitlines(True)
    line_endings = list(_find_line_endings(source))
    
    def _find_line_ending(position):
        for line_ending in line_endings:
            if line_ending >= position:
                return line_ending
        
        raise Exception("Could not find line ending")
    
    def _should_rewrite_import(name):
        return name.split(".")[0] in top_level_names
    
    def _generate_simple_import_replacement(node):
        temp_index = itertools.count()
        
        replacement = []
        
        for name in node.names:
            if _should_rewrite_import(name.name):
                parts = name.name.split(".")
                if name.asname is None:
                    for part_index, part in enumerate(parts):
                        if part_index == 0:
                            replacement.append("from ." + ("." * depth) + " import " + part)
                        else:
                            variable_name = "___vendorize__{0}".format(next(temp_index))
                            replacement.append(
                                "from ." + ("." * depth) + ".".join(parts[:part_index]) +
                                " import " + part +
                                " as " + variable_name)
                            replacement.append(".".join(parts[:part_index + 1]) + " = " + variable_name)
                else:
                    replacement.append(
                        "from ." + ("." * depth) + ".".join(parts[:-1]) +
                        " import " + parts[-1] +
                        " as " + name.asname)
            else:
                statement = "import " + name.name
                if name.asname is not None:
                    statement += " as " + name.asname
                replacement.append(statement)
        
        _, line_ending_col_offset = _find_line_ending((node.lineno, node.col_offset))
        return _Replacement(
            _Location(node.lineno, node.col_offset),
            # TODO: handle multi-line statements
            line_ending_col_offset - node.col_offset,
            "\n".join(replacement))
    
    def _generate_import_from_replacement(node):
        line = source_lines[node.lineno - 1]
        col_offset = node.col_offset
        from_keyword = "from"
        assert line[col_offset:col_offset + len(from_keyword)] == from_keyword
        col_offset += len(from_keyword)
        while line[col_offset].isspace():
            col_offset += 1
        return _Replacement(
            _Location(node.lineno, col_offset),
            0,
            "." + ("." * depth))
    
    replacements = []
    
    class ImportVisitor(ast.NodeVisitor):
        def visit_Import(self, node):
            if any(_should_rewrite_import(name.name) for name in node.names):
                replacements.append(_generate_simple_import_replacement(node))
            
        def visit_ImportFrom(self, node):
            if not node.level and _should_rewrite_import(node.module):
                replacements.append(_generate_import_from_replacement(node))
    
    python_ast = ast.parse(source)
    ImportVisitor().visit(python_ast)
    return _replace_strings(source, replacements)


def _find_line_endings(source):
    token_stream = tokenize.generate_tokens(io.StringIO(source + "\n").readline)
    for token_type, token_str, start, end, line in token_stream:
        if token_type == tokenize.NEWLINE:
            yield start


_Location = collections.namedtuple("_Location", ["lineno", "col_offset"])

_Replacement = collections.namedtuple("_Replacement", [
    "location",
    "length",
    "value"
])

def _replace_strings(source, replacements):
    lines = source.splitlines(True)
    
    replacements = sorted(replacements, key=lambda replacement: replacement.location, reverse=True)
    
    for replacement in replacements:
        line_index = replacement.location.lineno - 1
        col_offset = replacement.location.col_offset
        lines[line_index] = _str_replace(lines[line_index], replacement.length, col_offset, replacement.value)
    
    return "".join(lines)


def _str_replace(original, length, index, to_insert):
    return original[:index] + to_insert + original[index + length:]





import os

                    
def mkdir_p(path):
    if not os.path.exists(path):
        os.makedirs(path)


def ensure_file_exists(path):
    if not os.path.exists(path):
        mkdir_p(os.path.dirname(path))
        open(path, "w").close()



import sys

from . import vendorize_requirement, vendorize_requirements


def main():
    if len(sys.argv) < 2:
        vendorize_requirements(path="vendorize.toml")
    elif len(sys.argv) < 3:
        vendorize_requirements(path=sys.argv[1])
    else:
        requirement = sys.argv[1]
        target_directory = sys.argv[2]
        vendorize_requirement(cwd=".", requirement=requirement, target_directory=target_directory)


if __name__ == "__main__":
    main()



import csv
import io
import os
import pathlib
import subprocess

from . import python_source
from ._vendor import pytoml as toml
from .files import mkdir_p, ensure_file_exists
from .import_rewrite import rewrite_imports_in_module


def vendorize_requirements(path):
    with open(path) as fileobj:
        config = toml.load(fileobj)
    target_directory = os.path.join(os.path.dirname(path), config["target"])
    ensure_file_exists(os.path.join(target_directory, "__init__.py"))
    _download_requirements(
        cwd=os.path.dirname(path) or None,
        requirements=config["packages"],
        target_directory=target_directory,
    )
    top_level_names = _read_top_level_names(target_directory)
    _rewrite_imports(target_directory, top_level_names)


def vendorize_requirement(cwd, requirement, target_directory):
    _download_requirements(cwd=cwd, requirements=[requirement], target_directory=target_directory)
    top_level_names = _read_top_level_names(target_directory)
    _rewrite_imports(target_directory, top_level_names)


def _download_requirements(cwd, requirements, target_directory):
    mkdir_p(target_directory)
    subprocess.check_call(
        ["pip", "install", "--no-dependencies", "--target", target_directory] + requirements,
        cwd=cwd)

def _read_top_level_names(target_directory):
    top_level_names = set()

    for name in os.listdir(target_directory):
        if name.endswith(".dist-info"):
            path = os.path.join(target_directory, name)
            top_level_names.update(_read_top_level_names_from_dist_info(path))

        elif name.endswith(".egg-info"):
            path = os.path.join(target_directory, name)
            top_level_names.update(_read_top_level_names_from_egg_info(path))

    return top_level_names


def _read_top_level_names_from_dist_info(dist_info_path):
    path = os.path.join(dist_info_path, "RECORD")

    if not os.path.exists(path):
        return

    py_extension = ".py"

    with open(path, "rt", encoding="utf-8") as fileobj:
        for line in csv.reader(fileobj):
            if len(line) > 0:
                record_path = line[0]
                if record_path.endswith(py_extension):
                    top_part = pathlib.Path(record_path).parts[0]
                    if top_part.endswith(py_extension):
                        yield top_part[:-len(py_extension)]
                    else:
                        yield top_part


def _read_top_level_names_from_egg_info(egg_info_path):
    path = os.path.join(egg_info_path, "top_level.txt")
    if os.path.exists(path):
        with open(path) as top_level_file:
            return list(filter(None, map(lambda line: line.strip(), top_level_file)))


def _rewrite_imports(target_directory, top_level_names):
    for top_level_name in top_level_names:
        module_path = os.path.join(target_directory, top_level_name + ".py")
        if os.path.exists(module_path):
            _rewrite_imports_in_module(module_path, top_level_names, depth=0)

        package_path = os.path.join(target_directory, top_level_name)
        if os.path.exists(package_path):
            _rewrite_imports_in_package(package_path, top_level_names, depth=1)

def _rewrite_imports_in_package(package_path, top_level_names, depth):
    for name in os.listdir(package_path):
        child_path = os.path.join(package_path, name)
        if name.endswith(".py"):
            _rewrite_imports_in_module(child_path, top_level_names, depth=depth)

        if os.path.isdir(child_path):
            _rewrite_imports_in_package(child_path, top_level_names, depth=depth + 1)


def _rewrite_imports_in_module(module_path, top_level_names, depth):
    with io.open(module_path, "rb") as source_file:
        encoding = python_source.find_encoding(source_file)

    with io.open(module_path, "r", encoding=encoding, newline='') as source_file:
        source = source_file.read()

    rewritten_source = rewrite_imports_in_module(source, top_level_names, depth)

    with io.open(module_path, "w", encoding=encoding, newline='') as source_file:
        source_file.write(rewritten_source)

    pyc_path = os.path.splitext(module_path)[0] + ".pyc"
    if os.path.exists(pyc_path):
        os.unlink(pyc_path)




#!/usr/bin/env python

import os
import sys
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='vendorize',
    version='0.3.0',
    description='Vendorize packages from PyPI',
    long_description=read("README.rst"),
    author='Michael Williamson',
    author_email='mike@zwobble.org',
    url='http://github.com/mwilliamson/python-vendorize',
    packages=find_packages(exclude=["tests", "tests.*", "examples", "examples.*"]),
    entry_points={
        "console_scripts": [
            "python-vendorize=vendorize.cli:main"
        ]
    },
    keywords="vendor vendorize",
    license="BSD-2-Clause",
    python_requires=">=3.5",
)




import re


def find_encoding(fileobj):
    for line_index in range(0, 2):
        line = fileobj.readline()
        # From PEP 263
        match = re.match(br"^[ \t\f]*#.*?coding[:=][ \t]*([-_.a-zA-Z0-9]+)", line)
        if match is not None:
            return match.group(1).decode("ascii")

    return "utf-8"
